/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "rs485isolator.h"

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void commaunication_delay ( void );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void rs485isolator_cfg_setup ( rs485isolator_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->rx_pin = HAL_PIN_NC;
    cfg->tx_pin = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->re = HAL_PIN_NC;
    cfg->de   = HAL_PIN_NC;

    cfg->baud_rate  = 9600;
    cfg->data_bit   = UART_DEFAULT_DATA_BITS;
    cfg->parity_bit = UART_DEFAULT_PARITY_BIT;
    cfg->stop_bit   = UART_DEFAULT_STOP_BIT;
}

RS485ISOLATOR_RETVAL rs485isolator_init ( rs485isolator_t *ctx, rs485isolator_cfg_t *cfg )
{
    uart_config_t uart_cfg;

    uart_configure_default( &uart_cfg );
    uart_cfg.pins.tx_pin = cfg->tx_pin;
    uart_cfg.pins.rx_pin = cfg->rx_pin;
    uart_cfg.baud        = cfg->baud_rate;
    uart_cfg.data_bits   = cfg->data_bit;
    uart_cfg.parity      = cfg->parity_bit;
    uart_cfg.stop        = cfg->stop_bit;

    if ( uart_open( &ctx->uart, &uart_cfg ) != UART_SUCCESS )
    {
        return RS485ISOLATOR_INIT_ERROR;
    }

    // Output pins 

    digital_out_init( &ctx->re, cfg->re );
    digital_out_init( &ctx->de, cfg->de );
    
    rs485isolator_set_receiver_mode( ctx, RS485ISOLATOR_RECEIVER_ENABLE );
    rs485isolator_set_driver_mode( ctx, RS485ISOLATOR_DRIVER_DISABLE );

    return RS485ISOLATOR_OK;
}

void rs485isolator_generic_multi_write 
( 
    rs485isolator_t *ctx, 
    rs485isolator_data_t *data_buf,  
    uart_length_t len 
)
{
    uint8_t cnt;

    for ( cnt = 0; cnt < len; cnt++ )
    {
        rs485isolator_generic_single_write( ctx, data_buf[ cnt ] );
    }
}

rs485isolator_data_t rs485isolator_generic_single_read ( rs485isolator_t *ctx )
{
    rs485isolator_data_t rx_data;

    rx_data =  uart_read( &ctx->uart );

    return rx_data;
}

void rs485isolator_generic_single_write ( rs485isolator_t *ctx, rs485isolator_data_t tx_data )
{
    rs485isolator_set_receiver_mode( ctx, RS485ISOLATOR_RECEIVER_DISABLE );
    rs485isolator_set_driver_mode( ctx, RS485ISOLATOR_DRIVER_ENABLE );

    uart_write( &ctx->uart, tx_data );
    commaunication_delay( );

    rs485isolator_set_receiver_mode( ctx, RS485ISOLATOR_RECEIVER_ENABLE );
    rs485isolator_set_driver_mode( ctx, RS485ISOLATOR_DRIVER_DISABLE );
}

void rs485isolator_set_receiver_mode ( rs485isolator_t *ctx, uint8_t state )
{
    digital_out_write( &ctx->re, state );
}

void rs485isolator_set_driver_mode ( rs485isolator_t *ctx, uint8_t state )
{
    digital_out_write( &ctx->de, state );
}


// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void commaunication_delay ( void )
{
    Delay_10ms( );
}

// ------------------------------------------------------------------------- END

